Package: bSims
Type: Package
Title: Agent-Based Bird Point Count Simulator
Version: 0.3-3
Date: 2025-07-12
Authors@R: 
    c(person(given = "Peter",
             family = "Solymos",
             comment = c(ORCID = "0000-0001-7337-1740"),
             role = c("aut", "cre"),
             email = "psolymos@gmail.com"))
Maintainer: Peter Solymos <psolymos@gmail.com>
Description: A highly scientific and utterly addictive 
  bird point count simulator
  to test statistical assumptions, aid survey design,
  and have fun while doing it (Solymos 2024 <doi:10.1007/s42977-023-00183-2>).
  The simulations follow time-removal and distance sampling models 
  based on Matsuoka et al. (2012) <doi:10.1525/auk.2012.11190>,
  Solymos et al. (2013) <doi:10.1111/2041-210X.12106>,
  and Solymos et al. (2018) <doi:10.1650/CONDOR-18-32.1>,
  and sound attenuation experiments by 
  Yip et al. (2017) <doi:10.1650/CONDOR-16-93.1>.
License: GPL-2
LazyLoad: yes
Depends: intrval, mefa4, MASS, deldir (>= 1.0-2)
Imports: parallel, pbapply
URL: https://github.com/psolymos/bSims
BugReports: https://github.com/psolymos/bSims/issues
Suggests: knitr, rmarkdown, detect, shiny
VignetteBuilder: knitr
Language: en-US
NeedsCompilation: no
Packaged: 2025-07-13 22:30:59 UTC; Peter
Author: Peter Solymos [aut, cre] (ORCID:
    <https://orcid.org/0000-0001-7337-1740>)
Repository: CRAN
Date/Publication: 2025-07-13 22:50:06 UTC
Built: R 4.6.0; ; 2026-04-26 02:58:05 UTC; unix
